# bin/agentctl.py

def main(args, syscall):
    """
    Controller for agent modules.
    For future: agentctl start browser-agent
    """
    syscall.log("[agentctl] Module system is not implemented yet.")
    return "agentctl: stub ready"
# --- IGNORE ---